# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
# Copyright (C) 2020 Northwestern University.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.


"""Draft File Service."""


from invenio_records_resources.services import FileService, FileServiceConfig


class DraftFileServiceConfig(FileServiceConfig):
    """Draft File Service configuration."""

    pass


class DraftFileService(FileService):
    """Draft File Service."""

    default_config = DraftFileServiceConfig

    # High-level API
    def read(self, id_, identity, *args, **kwargs):
        """Retrieve the actual file."""
        # TODO: IMPLEMENT ME!
        # High likelihood that we can just rely on FileService here
        # and simply configure DraftFileServiceConfig appropriately
        return self.resource_unit_cls()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
# Copyright (C) 2020 Northwestern University.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Invenio Deposits Resources."""

from flask_resources import CollectionResource
from flask_resources.resources import ResourceConfig

# TODO: Get rid of them when implementation is done
STUB_ITEM_RESULT = ({"TODO": "IMPLEMENT ME"}, 200)
STUB_LIST_RESULT = ([{"TODO": "IMPLEMENT ME"}], 200)


# Proposal: "Deposits" is the term to talk about entities that are either
#           draft or published records
class DepositResourceConfig(ResourceConfig):
    """Deposit resource config."""

    list_route = "/user/records"


class DepositResource(CollectionResource):
    """Deposit resource."""

    default_config = DepositResourceConfig

    def search(self, *args, **kwargs):
        """Perform a search over the items."""
        return STUB_LIST_RESULT



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Pytest configuration.

See https://pytest-invenio.readthedocs.io/ for documentation on which test
fixtures are available.
"""

import pytest
from flask_principal import Identity
from invenio_access import any_user
from invenio_app.factory import create_api
from invenio_db import db
from invenio_records.api import Record
from invenio_records.models import RecordMetadataBase
from invenio_records_permissions.generators import AnyUser
from invenio_records_permissions.policies.records import RecordPermissionPolicy
from invenio_records_resources.resources import RecordResource
from invenio_records_resources.services import RecordService, \
    RecordServiceConfig

from invenio_drafts_resources.drafts import DraftBase, DraftMetadataBase
from invenio_drafts_resources.resources import DraftActionResource, \
    DraftResource
from invenio_drafts_resources.services import RecordDraftService, \
    RecordDraftServiceConfig


class AnyUserPermissionPolicy(RecordPermissionPolicy):
    """Custom permission policy."""

    can_list = [AnyUser()]
    can_create = [AnyUser()]
    can_read = [AnyUser()]
    can_update = [AnyUser()]
    can_delete = [AnyUser()]
    can_read_files = [AnyUser()]
    can_update_files = [AnyUser()]
    can_publish = [AnyUser()]


class CustomDraftMetadata(db.Model, DraftMetadataBase):
    """Represent a custom draft metadata."""

    __tablename__ = 'custom_drafts_metadata'


class CustomDraft(DraftBase):
    """Custom draft API."""

    model_cls = CustomDraftMetadata


class CustomRecordMetadata(db.Model, RecordMetadataBase):
    """Represent a custom draft metadata."""

    __tablename__ = 'custom_record_metadata'


class CustomRecord(Record):
    """Custom draft API."""

    model_cls = CustomRecordMetadata


class CustomRecordServiceConfig(RecordServiceConfig):
    """Custom draft service config."""

    record_cls = CustomRecord
    permission_policy_cls = AnyUserPermissionPolicy


class CustomRecordDraftServiceConfig(RecordDraftServiceConfig):
    """Custom draft service config."""

    draft_cls = CustomDraft
    record_cls = CustomRecord
    permission_policy_cls = AnyUserPermissionPolicy


@pytest.fixture(scope='module')
def app_config(app_config):
    """Override pytest-invenio app_config fixture.

    For test purposes we need to enforce some configuration variables before
    endpoints are created.
    invenio-records-rest is imported from invenio-records-permissions, so
    we need to disable its default endpoint, until we are completely
    decoupled from invenio-records-rest. Issue:
    https://github.com/inveniosoftware/invenio-records-permissions/issues/51
    """
    app_config["RECORDS_REST_ENDPOINTS"] = {}

    return app_config


@pytest.fixture(scope="module")
def create_app(instance_path):
    """Application factory fixture."""
    return create_api


def _draft_service():
    """Create a draft service."""
    return RecordDraftService(config=CustomRecordDraftServiceConfig)


def _record_service():
    """Create a record service."""
    return RecordService(config=CustomRecordServiceConfig)


@pytest.fixture(scope="module")
def app(app):
    """Application factory fixture."""
    with app.app_context():
        record_bp = RecordResource(
            service=_draft_service()
        ).as_blueprint("record_resource")
        draft_bp = DraftResource(
            service=_draft_service()
        ).as_blueprint("draft_resource")
        draft_action_bp = DraftActionResource(
            service=_draft_service()
        ).as_blueprint("draft_action_resource")

        app.register_blueprint(record_bp)
        app.register_blueprint(draft_bp)
        app.register_blueprint(draft_action_bp)
        yield app


@pytest.fixture(scope="module")
def draft_service():
    """Draft service factory fixture."""
    return _draft_service()


@pytest.fixture(scope="module")
def record_service():
    """Record service factory fixture."""
    return _record_service()


@pytest.fixture(scope="function")
def input_draft():
    """Minimal draft data as dict coming from the external world."""
    return {
        "_access": {"metadata_restricted": False, "files_restricted": False},
        "_owners": [1],
        "_created_by": 1
    }


@pytest.fixture(scope="function")
def input_record():
    """Minimal record data as dict coming from the external world."""
    return {
        "_access": {"metadata_restricted": False, "files_restricted": False},
        "_owners": [1],
        "_created_by": 1,
        "title": "A Romans story",
        "description": "A looong description full of lorem ipsums"
    }


@pytest.fixture(scope="function")
def fake_identity():
    """Fake identity providing `any_user` system role."""
    identity = Identity(1)
    identity.provides.add(any_user)

    return identity



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Draft API."""

from invenio_db import db
from invenio_records.api import Record


class DraftBase(Record):
    """Draft base API for metadata creation and manipulation."""

    # WHY: We want to force the model_cls to be specified by the user
    # No default one is given, only the base.
    model_cls = None
    default_status = 'draft'

    @property
    def expiry_date(self):
        """Get model identifier."""
        return self.model.expiry_date if self.model else None

    @property
    def status(self):
        """Get revision identifier."""
        return self.model.status if self.model else self.default_status

    @property
    def fork_id(self):
        """Get revision identifier."""
        return self.model.fork_id if self.model else None

    @classmethod
    def create(cls, data, record=None, **kwargs):
        """Create a new draft instance and store it in the database."""
        with db.session.begin_nested():
            draft = cls(data)

            draft.validate(**kwargs)

            draft.model = cls.model_cls(
                id=record.id if record else None,
                fork_version_id=record.revision_id if record else None,
                expiry_date=draft.expiry_date,
                status=draft.status,
                json=draft,
            )

            db.session.add(draft.model)

        return draft



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
# Copyright (C) 2020 Northwestern University.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Invenio Drafts Resources module to create REST APIs."""

from flask import g
from flask_resources import CollectionResource, SingletonResource
from flask_resources.context import resource_requestctx
from flask_resources.resources import ResourceConfig
from invenio_records_resources.responses import RecordResponse
from invenio_records_resources.schemas import RecordSchemaJSONV1
from invenio_records_resources.serializers import RecordJSONSerializer

from ..serializers import DraftJSONSerializer
from ..services import DraftVersionService, RecordDraftService
from ..services.schemas import DraftSchemaJSONV1


class DraftResourceConfig(ResourceConfig):
    """Draft resource config."""

    list_route = "/records/<pid_value>/draft"
    response_handlers = {
        "application/json": RecordResponse(
            DraftJSONSerializer(schema=DraftSchemaJSONV1)
        )
    }


class DraftResource(SingletonResource):
    """Draft resource."""

    default_config = DraftResourceConfig

    def __init__(self, service=None, *args, **kwargs):
        """Constructor."""
        super(DraftResource, self).__init__(*args, **kwargs)
        self.service = service or RecordDraftService()

    def read(self, *args, **kwargs):
        """Read an item."""
        identity = g.identity
        id_ = resource_requestctx.route["pid_value"]

        return self.service.read_draft(id_, identity), 200

    def create(self, *args, **kwargs):
        """Create an item."""
        data = resource_requestctx.request_content
        identity = g.identity
        id_ = resource_requestctx.route["pid_value"]

        return self.service.edit(id_, data, identity), 201

    def update(self, *args, **kwargs):
        """Update an item."""
        # TODO: IMPLEMENT ME!
        return self.service.update(), 200

    def delete(self, *args, **kwargs):
        """Delete an item."""
        # TODO: IMPLEMENT ME!
        return self.service.delete(), 200


class DraftVersionResourceConfig(ResourceConfig):
    """Draft resource config."""

    list_route = "/records/<pid_value>/versions"


class DraftVersionResource(CollectionResource):
    """Draft version resource."""

    default_config = DraftVersionResourceConfig

    def __init__(self, service=None, *args, **kwargs):
        """Constructor."""
        super(DraftVersionResource, self).__init__(*args, **kwargs)
        self.service = service or DraftVersionService()

    def search(self, *args, **kwargs):
        """Perform a search over the items."""
        # TODO: IMPLEMENT ME!
        return self.service.search()

    def create(self, *args, **kwargs):
        """Create an item."""
        # TODO: IMPLEMENT ME!
        return self.service.create()


class DraftActionResourceConfig(ResourceConfig):
    """Draft action resource config."""

    list_route = "/records/<pid_value>/draft/actions/<action>"
    response_handlers = {
        "application/json": RecordResponse(
            RecordJSONSerializer(schema=RecordSchemaJSONV1)
        )
    }


class DraftActionResource(SingletonResource):
    """Draft action resource."""

    default_config = DraftActionResourceConfig

    def __init__(self, service=None, *args, **kwargs):
        """Constructor."""
        super(DraftActionResource, self).__init__(*args, **kwargs)
        self.service = service or RecordDraftService()

    def create(self, *args, **kwargs):
        """Any POST business logic."""
        if resource_requestctx.route["action"] == "publish":
            identity = g.identity
            id_ = resource_requestctx.route["pid_value"]
            return self.service.publish(id_, identity), 200
        return {}, 200



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
# Copyright (C) 2020 Northwestern University.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Invenio-Drafts-Resources Configuration."""

DRAFTS_RESOURCES_LINK_URLS = {
    'record': '{base}/records/{pid}',
    'records': '{base}/records/',
    'draft': '{base}/records/{pid}/draft/',
}



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Draft Models API."""

import uuid
from datetime import datetime

from invenio_db import db
from invenio_records.models import Timestamp
from sqlalchemy.dialects import mysql, postgresql
from sqlalchemy_utils.types import JSONType, UUIDType


class DraftMetadataBase(Timestamp):
    """Represent a base class for draft metadata.

    The DraftMetadata object  contains a `created` and  a `updated`
    properties that are automatically updated.
    """

    # Enables SQLAlchemy-Continuum versioning
    __versioned__ = {}

    id = db.Column(
        UUIDType,
        primary_key=True,
        default=uuid.uuid4,
    )
    """Draft identifier, same as the record it drafts."""

    fork_version_id = db.Column(db.Integer)
    """Version id of the record it is draft of."""

    version_id = db.Column(db.Integer, nullable=False)
    """Used by SQLAlchemy for optimistic concurrency control."""

    status = db.Column(db.String(255), default="draft", nullable=False)
    """Status for workflow management."""

    expiry_date = db.Column(
        db.DateTime().with_variant(mysql.DATETIME(fsp=6), "mysql"),
        default=datetime.utcnow,
        nullable=True
    )
    """Specifies when the draft expires. If `NULL` the draft doesn't expire."""

    json = db.Column(
        db.JSON().with_variant(
            postgresql.JSONB(none_as_null=True),
            'postgresql',
        ).with_variant(
            JSONType(),
            'sqlite',
        ).with_variant(
            JSONType(),
            'mysql',
        ),
        default=lambda: dict(),
        nullable=True
    )
    """Store metadata in JSON format.
    When you create a new `Record the `json field value should never be
    `NULL`. Default value is an empty dict. `NULL` value means that the
    record metadata has been deleted.
    """

    __mapper_args__ = {
        'version_id_col': version_id
    }



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
# Copyright (C) 2020 Northwestern University.
#
# Invenio-Records-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Resource units."""

from invenio_records_resources.resource_units import IdentifiedRecord


class IdentifiedRecordDraft(IdentifiedRecord):
    """Resource unit representing pid + Record data clump.

    Since a draft can be created for a non-existing record,
    it might not have a PID.
    """

    def __init__(self, pid=None, record=None):
        """Initialize the record state."""
        self.id = pid.pid_value if pid else None
        self.pids = [pid] if pid else []
        self.record = record



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
# Copyright (C) 2020 Northwestern University.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Draft Services."""

from .draft import RecordDraftService, RecordDraftServiceConfig
from .draft_file import DraftFileService, DraftFileServiceConfig
from .draft_file_metadata import DraftFileMetadataService, \
    DraftFileMetadataServiceConfig
from .draft_version import DraftVersionService, DraftVersionServiceConfig

__all__ = (
    "DraftFileMetadataService",
    "DraftFileMetadataServiceConfig",
    "DraftFileService",
    "DraftFileServiceConfig",
    "RecordDraftService",
    "RecordDraftServiceConfig",
    "DraftVersionService",
    "DraftVersionServiceConfig"
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
# Copyright (C) 2020 Northwestern University.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.


"""Draft Service."""

from .draft import RecordDraftService, RecordDraftServiceConfig


class DraftVersionServiceConfig(RecordDraftServiceConfig):
    """Draft Version Service configuration."""

    # Service configuration
    # TODO: FILL ME!

    # RecordService configuration
    # TODO: FILL ME!

    # DraftVersionService configuration
    # TODO: FILL ME!


class DraftVersionService(RecordDraftService):
    """Draft Service interface."""

    default_config = DraftVersionServiceConfig

    # High-level API
    def search(self, querystring, identity, pagination=None, *args, **kwargs):
        """Search for drafts matching the querystring."""
        # TODO: IMPLEMENT ME!
        return self.resource_list_cls()

    def create(self, data, identity):
        """Create a draft."""
        # TODO: IMPLEMENT ME!
        return self.resource_unit_cls()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
#
# Invenio-Records-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Marshmallow JSON schema."""


from marshmallow import INCLUDE, Schema, fields


class DraftMetadataSchemaJSONV1(Schema):
    """Basic metadata schema class."""

    class Meta:
        """Meta class to accept unknwon fields."""

        unknown = INCLUDE

    _created_by = fields.Integer(required=True)


class DraftSchemaJSONV1(Schema):
    """Schema for drafts v1 in JSON."""

    # TODO: Remove this comment
    # attribute="pid.pid_value" is not needed anymore since it is always `id`
    # PID resolving happends only at controller level.
    id = fields.String(attribute="pid.pid_value")
    metadata = fields.Nested(DraftMetadataSchemaJSONV1)
    links = fields.Raw()
    created = fields.Str()
    updated = fields.Str()
    status = fields.Str()
    expiry_date = fields.Str()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Invenio Drafts Resources module to create REST APIs"""

import json

import pytest
from sqlalchemy.orm.exc import NoResultFound

from invenio_drafts_resources.resources import DraftActionResource, \
    DraftActionResourceConfig, DraftResource, DraftResourceConfig, \
    DraftVersionResource, DraftVersionResourceConfig

HEADERS = {"content-type": "application/json", "accept": "application/json"}


def test_create_draft_of_new_record(client, draft_service, input_draft,
                                    fake_identity):
    """Test draft creation of a non-existing record."""
    response = client.post(
        "/records", data=json.dumps(input_draft), headers=HEADERS
    )

    assert response.status_code == 201
    response_fields = response.json.keys()
    fields_to_check = ['pid', 'metadata', 'revision',
                       'created', 'updated', 'links']

    for field in fields_to_check:
        assert field in response_fields


def test_create_draft_of_existing_record(client, record_service,
                                         input_record, fake_identity):
    """Test draft creation of an existing record."""
    # Create new record manually since the endpoint it overwritten
    identified_record = record_service.create(
        data=input_record, identity=fake_identity
    )

    recid = identified_record.id
    assert recid

    for key, value in input_record.items():
        assert identified_record.record[key] == value

    # Create new draft of said record
    orig_title = input_record['title']
    input_record['title'] = "Edited title"
    response = client.post(
        "/records/{}/draft".format(recid),
        data=json.dumps(input_record),
        headers=HEADERS
    )

    assert response.status_code == 201
    response_fields = response.json.keys()
    fields_to_check = ['pid', 'metadata', 'revision',
                       'created', 'updated', 'links']

    for field in fields_to_check:
        assert field in response_fields

    assert response.json['metadata']['title'] == input_record['title']

    # Check the actual record was not modified
    response = client.get(
        "/records/{}".format(recid),
        headers=HEADERS
    )

    assert response.status_code == 200
    response_fields = response.json.keys()
    fields_to_check = ['pid', 'metadata', 'revision',
                       'created', 'updated', 'links']

    for field in fields_to_check:
        assert field in response_fields

    assert response.json['metadata']['title'] == orig_title


def test_publish_draft_of_new_record(client, draft_service, input_record,
                                     fake_identity):
    """Test draft publication of a non-existing record.

    It has to first create said draft.
    """
    # Create the draft
    response = client.post(
        "/records", data=json.dumps(input_record), headers=HEADERS
    )

    assert response.status_code == 201
    recid = response.json['pid']

    # Publish it
    response = client.post(
        "/records/{}/draft/actions/publish".format(recid), headers=HEADERS
    )

    assert response.status_code == 200
    response_fields = response.json.keys()
    fields_to_check = ['pid', 'metadata', 'revision',
                       'created', 'updated', 'links']

    for field in fields_to_check:
        assert field in response_fields

    # Check draft deletion
    # TODO: Remove import when exception is properly handled
    with pytest.raises(NoResultFound):
        response = client.get(
            "/records/{}/draft".format(recid),
            headers=HEADERS
        )
    # assert response.status_code == 404

    # Test record exists
    response = client.get("/records/{}".format(recid), headers=HEADERS)

    assert response.status_code == 200

    response_fields = response.json.keys()
    fields_to_check = ['pid', 'metadata', 'revision',
                       'created', 'updated', 'links']

    for field in fields_to_check:
        assert field in response_fields



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Invenio Drafts Resources module to create REST APIs"""

import pytest
from invenio_pidstore.models import PIDStatus
from sqlalchemy.orm.exc import NoResultFound


def test_create_draft_of_new_record(app, draft_service, input_draft,
                                    fake_identity):
    """Test draft creation of a non-existing record."""
    # Needs `app` context because of invenio_access/permissions.py#166
    identified_draft = draft_service.create(
        data=input_draft, identity=fake_identity
    )

    assert identified_draft.id

    for key, value in input_draft.items():
        assert identified_draft.record[key] == value


def test_create_draft_of_existing_record(app, draft_service, record_service,
                                         input_record, fake_identity):
    """Test draft creation of an existing record."""
    # Needs `app` context because of invenio_access/permissions.py#166
    # Create new record
    identified_record = record_service.create(
        data=input_record, identity=fake_identity
    )

    recid = identified_record.id
    assert recid

    for key, value in input_record.items():
        assert identified_record.record[key] == value

    # Create new draft of said record
    orig_title = input_record['title']
    input_record['title'] = "Edited title"
    identified_draft = draft_service.edit(
        data=input_record,
        identity=fake_identity,
        id_=recid
    )

    assert identified_draft.id == recid

    for key, value in input_record.items():
        assert identified_draft.record[key] == value

    # Check the actual record was not modified
    identified_record = draft_service.read(id_=recid, identity=fake_identity)

    assert identified_record.record['title'] == orig_title


def test_publish_draft_of_new_record(app, draft_service, input_record,
                                     fake_identity):
    """Test draft publication of a non-existing record.

    It has to first create said draft.
    """
    # Needs `app` context because of invenio_access/permissions.py#166
    # Crate the draft
    identified_draft = draft_service.create(
        data=input_record, identity=fake_identity
    )
    assert identified_draft.id
    for pid in identified_draft.pids:
        assert pid.status == PIDStatus.NEW

    for key, value in input_record.items():
        assert identified_draft.record[key] == value

    # Publish it
    identified_record = draft_service.publish(
        id_=identified_draft.id, identity=fake_identity
    )

    assert identified_record.id
    for pid in identified_record.pids:
        assert pid.status == PIDStatus.REGISTERED

    for key, value in input_record.items():
        assert identified_record.record[key] == value

    # Check draft deletion
    # TODO: Remove import when exception is properly handled
    with pytest.raises(NoResultFound):
        identified_draft = draft_service.read_draft(
            identified_draft.id, identity=fake_identity
        )

    # Test record exists
    identified_record = draft_service.read(
        identified_record.id, identity=fake_identity
    )

    assert identified_record.id
    for pid in identified_record.pids:
        assert pid.status == PIDStatus.REGISTERED

    for key, value in input_record.items():
        assert identified_record.record[key] == value



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
# Copyright (C) 2020 Northwestern University.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.


"""Draft File Service."""

from invenio_records_resources.services import FileMetadataService, \
    FileMetadataServiceConfig


class DraftFileMetadataServiceConfig(FileMetadataServiceConfig):
    """Draft File Metadata Service configuration."""

    # TODO: FILL ME!
    pass


class DraftFileMetadataService(FileMetadataService):
    """Draft File Metadata Service."""

    default_config = DraftFileMetadataServiceConfig

    # High-level API
    def search(self, querystring, identity, pagination=None, *args, **kwargs):
        """Search for drafts matching the querystring."""
        # TODO: IMPLEMENT ME!
        # High likelihood that we can just rely on RecordService here
        # and simply configure DraftServiceConfig appropriately
        return self.resource_list_cls()

    def create(self, data, identity, *args, **kwargs):
        """Create a draft."""
        # TODO: IMPLEMENT ME!
        return self.resource_unit_cls()

    def update_all(self, data, identity, *args, **kwargs):
        """Delete an item."""
        # TODO: IMPLEMENT ME!
        return self.resource_list_cls()

    def delete_all(self, data, identity, *args, **kwargs):
        """Delete an item."""
        # TODO: IMPLEMENT ME!
        return self.resource_list_cls()

    def read(self, id_, identity, *args, **kwargs):
        """Read an item."""
        # TODO: IMPLEMENT ME!
        return self.resource_unit_cls()

    def update(self, id_, data, identity, *args, **kwargs):
        """Replace a draft."""
        # TODO: IMPLEMENT ME!
        return self.resource_unit_cls()

    def delete(self, id_, identity, *args, **kwargs):
        """Delete an item."""
        # TODO: IMPLEMENT ME!
        return self.resource_unit_cls()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Invenio Drafts Resources module to create REST APIs"""


from invenio_drafts_resources.resources import DraftFileActionResource, \
    DraftFileActionResourceConfig, DraftFileResource, \
    DraftFileResourceConfig

HEADERS = {"content-type": "application/json", "accept": "application/json"}

# TODO: IMPLEMENT ME!



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Invenio Drafts Resources module to create REST APIs"""

import os

from setuptools import find_packages, setup

readme = open("README.rst").read()
history = open("CHANGES.rst").read()

tests_require = [
    "pytest-invenio>=1.3.2",
    "invenio-app>=1.3.0",
]

# Should follow inveniosoftware/invenio versions
invenio_search_version = '>=1.2.0,<2.0.0'
invenio_db_version = '>=1.0.4,<2.0.0'

extras_require = {
    "docs": ["Sphinx>=1.5.1,<3"],
    # Elasticsearch version
    'elasticsearch6': [
        'invenio-search[elasticsearch6]{}'.format(invenio_search_version),
    ],
    'elasticsearch7': [
        'invenio-search[elasticsearch7]{}'.format(invenio_search_version),
    ],
    # Databases
    'mysql': [
        'invenio-db[mysql,versioning]{}'.format(invenio_db_version),
    ],
    'postgresql': [
        'invenio-db[postgresql,versioning]{}'.format(invenio_db_version),
    ],
    'sqlite': [
        'invenio-db[versioning]{}'.format(invenio_db_version),
    ],
    "tests": tests_require,
}

extras_require["all"] = []
for reqs in extras_require.values():
    extras_require["all"].extend(reqs)

setup_requires = [
    "Babel>=1.3",
    "pytest-runner>=3.0.0,<5",
]

install_requires = [
    "Flask-BabelEx>=0.9.4",
    "invenio-base>=1.2.3",
    "invenio-pidstore>=1.2.0",
    "invenio-indexer>=1.1.1",
    "invenio-records>=1.3.2",
    "invenio-rest>=1.2.1",
    # "flask-resources", # FIXME: Currently in dev
    # Service
    "invenio-accounts>=1.3.0",
    "invenio-files-rest>=1.2.0",
    "invenio-records-permissions>=0.8.0",
]

packages = find_packages()


# Get the version string. Cannot be done with import!
g = {}
with open(os.path.join("invenio_drafts_resources", "version.py"), "rt") as fp:
    exec(fp.read(), g)
    version = g["__version__"]

setup(
    name="Invenio-Drafts-Resources",
    version=version,
    description=__doc__,
    long_description=readme + "\n\n" + history,
    keywords="invenio TODO",
    license="MIT",
    author="CERN",
    author_email="info@inveniosoftware.org",
    url="https://github.com/inveniosoftware/Invenio-Drafts-Resources",
    packages=packages,
    zip_safe=False,
    include_package_data=True,
    platforms="any",
    entry_points={
        "invenio_base.apps": [
            "invenio_drafts_resources = invenio_drafts_resources:InvenioDraftsResources",
        ],
        "invenio_base.api_apps": [
            "invenio_drafts_resources = invenio_drafts_resources:InvenioDraftsResources",
        ],
        'invenio_config.module': [
            'invenio_drafts_resources = invenio_drafts_resources.config',
        ],
        "invenio_i18n.translations": ["messages = invenio_drafts_resources",],
        'invenio_db.models': [
            'invenio_drafts_resources = invenio_drafts_resources.drafts.models',
        ],
    },
    extras_require=extras_require,
    install_requires=install_requires,
    setup_requires=setup_requires,
    tests_require=tests_require,
    classifiers=[
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Development Status :: 1 - Planning",
    ],
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
# Copyright (C) 2020 Northwestern University.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.


"""Draft Service."""

from invenio_db import db
from invenio_pidstore.models import PIDStatus
from invenio_pidstore.providers.recordid_v2 import RecordIdProviderV2
from invenio_records_resources.services import MarshmallowDataValidator, \
    RecordService, RecordServiceConfig

from ..resource_units import IdentifiedRecordDraft
from .permissions import DraftPermissionPolicy
from .schemas import DraftMetadataSchemaJSONV1


class RecordDraftServiceConfig(RecordServiceConfig):
    """Draft Service configuration."""

    # Service configuration
    permission_policy_cls = DraftPermissionPolicy

    # RecordService configuration
    resource_unit_cls = IdentifiedRecordDraft

    # DraftService configuration.
    # WHY: We want to force user input choice here.
    draft_cls = None
    draft_data_validator = MarshmallowDataValidator(
        schema=DraftMetadataSchemaJSONV1
    )


class RecordDraftService(RecordService):
    """Draft Service interface."""

    RecordIdProviderV2.default_status_with_obj = PIDStatus.NEW
    default_config = RecordDraftServiceConfig

    # Over write resolver because default status is a class attr
    @property
    def resolver(self):
        """Factory for creating a draft resolver instance."""
        return self.config.resolver_cls(
            pid_type=self.config.resolver_pid_type,
            getter=self.record_cls.get_record,
            registered_only=False
        )

    # Draft attrs
    @property
    def draft_cls(self):
        """Factory for creating a record class."""
        return self.config.draft_cls

    @property
    def draft_data_validator(self):
        """Returns an instance of the draft data validator."""
        return self.config.draft_data_validator

    @property
    def draft_resolver(self):
        """Factory for creating a draft resolver instance."""
        return self.config.resolver_cls(
            pid_type=self.config.resolver_pid_type,
            getter=self.draft_cls.get_record,
            registered_only=False
        )

    def resolve_draft(self, id_):
        """Resolve a persistent identifier to a record draft."""
        return self.draft_resolver.resolve(id_)

    # High-level API
    # Inherits record read, search, create, delete and update

    def read_draft(self, id_, identity):
        """Retrieve a record."""
        pid, record = self.resolve_draft(id_)
        self.require_permission(identity, "read", record=record)
        # Todo: how do we deal with tombstone pages
        return self.resource_unit(pid=pid, record=record)

    def create(self, data, identity):
        """Create a draft for a new record.

        It does not eagerly create the associated record.
        """
        self.require_permission(identity, "create")
        validated_data = self.draft_data_validator.validate(data)
        draft = self.draft_cls.create(validated_data)
        # Create PID (New) associated to the draft UUID
        pid = self.minter(record_uuid=draft.id, data=draft)

        db.session.commit()  # Persist DB
        if self.indexer:
            self.indexer.index(draft)

        return self.config.resource_unit_cls(pid=pid, record=draft)

    def edit(self, id_, data, identity):
        """Create a draft for an existing record.

        :param id_: record PID value.
        """
        pid, record = self.resolve(id_)
        self.require_permission(identity, "create")
        validated_data = self.draft_data_validator.validate(data)
        draft = self.draft_cls.create(validated_data, record)
        db.session.commit()  # Persist DB
        if self.indexer:
            self.indexer.index(draft)

        return self.config.resource_unit_cls(pid=pid, record=draft)

    def publish(self, id_, identity):
        """Publish a draft."""
        self.require_permission(identity, "publish")
        # Get draft
        pid, draft = self.resolve_draft(id_=id_)
        # Validate and create record, register PID
        data = draft.dumps()
        self.data_validator.validate(data)  # Validate against record schema
        record = self.record_cls.create(data, id_=draft.id)  # Use same UUID
        pid.register()
        # Remove draft
        draft.delete()
        db.session.commit()  # Persist DB
        # Index the record
        if self.indexer:
            self.indexer.index(record)
            self.indexer.delete(draft)

        return self.resource_unit(pid=pid, record=record)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Version information for Invenio-Drafts-Resources.

This file is imported by ``invenio_drafts_resources.__init__``,
and parsed by ``setup.py``.
"""

__version__ = "0.1.0"



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Invenio Resources module to create REST APIs."""

import json

import pytz
from flask_resources.serializers import SerializerMixin
from invenio_records_resources.links import link_for
from invenio_records_resources.serializers import RecordJSONSerializer


class DraftJSONSerializer(RecordJSONSerializer):
    """Drafts JSON serializer implementation."""

    def __init__(self, schema=None):
        """Constructor."""
        self.schema = schema

    def _process_draft(self, draft_unit, *args, **kwargs):
        pid = draft_unit.id
        draft = draft_unit.record  # FIXME: refactor to unit
        draft_dict = dict(
            pid=pid,
            metadata=draft.dumps(),
            revision=draft.revision_id,
            status=draft.status,
            created=(
                pytz.utc.localize(draft.created).isoformat()
                if draft.created and not draft.created.tzinfo
                else None
            ),
            updated=(
                pytz.utc.localize(draft.updated).isoformat()
                if draft.updated and not draft.updated.tzinfo
                else None
            ),
            expiry=(
                pytz.utc.localize(draft.expiry_date).isoformat()
                if draft.expiry_date and not draft.expiry_date.tzinfo
                else None
            ),
            links=dict(
                self=link_for(api=True, tpl_key='draft', pid=pid),
                self_html=link_for(api=False, tpl_key='draft', pid=pid),
            )
        )

        # TODO: Shall we includ fork_version_id and record_pid in
        # the serialization?
        return draft_dict

    def serialize_object(self, obj, response_ctx=None, *args, **kwargs):
        """Dump the object into a json string."""
        if obj:  # e.g. delete op has no return body
            return json.dumps(self._process_draft(obj))
        else:
            return ""

    def serialize_object_list(
        self, obj_list, response_ctx=None, *args, **kwargs
    ):
        """Not implemented.

        Drafts are only serialized and returned back as individual objects.
        """
        raise NotImplementedError()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
# Copyright (C) 2020 Northwestern University.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Invenio Drafts Resources module to create REST APIs."""

from .deposit import DepositResource, DepositResourceConfig
from .draft import DraftActionResource, DraftActionResourceConfig, \
    DraftResource, DraftResourceConfig, DraftVersionResource, \
    DraftVersionResourceConfig
from .draft_file import DraftFileActionResource, \
    DraftFileActionResourceConfig, DraftFileResource, \
    DraftFileResourceConfig

__all__ = (
    "DepositResource",
    "DepositResourceConfig",
    "DraftResource",
    "DraftResourceConfig",
    "DraftActionResource",
    "DraftActionResourceConfig",
    "DraftVersionResource",
    "DrafVersiontResourceConfig",
    "DraftFileActionResourceConfig",
    "DraftFileActionResource",
    "DraftFileResourceConfig",
    "DraftFileResource",
)
